/*
 * $Id: routing_table.c,v 1.3 1998/07/21 21:22:49 labovit Exp $
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <netdb.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>

#include <mrt.h>
#include <version.h>
#include <array.h>
#include <hash.h>
#include <bgp.h>
#include <io.h>

static bgp_peer_t *create_fake_peer ();

/*
 * 2  view # | 
 * 
 *  4 prefix | 1 mask | status (VRTS_SUPPRESS) | 4 time originated | 
 *  4 len | attributes  
 */

void print_routing_table_msg (mrt_msg_t *msg) {
  char *stime, **cpp, *cp, date[MAXLINE];
  char mask, status;
  u_short view,attrlen, seq_num;
  u_long prefix_long, originated, t;
  bgp_peer_t *peer;
  prefix_t *prefix;
  int len;
  char *_sorigins[] = {"i", "e", "?", "a"};

  len = msg->length;
  cp = msg->value;
  UTIL_GET_SHORT (view, cp);
  UTIL_GET_SHORT (seq_num, cp);
  len -= 2;

  stime = my_strftime (msg->tstamp, "%D %T");
  if (seq_num == 0) {
    printf ("TIME: %s\n", stime);
    printf ("TYPE: %s", S_MRT_MSG_TYPES[msg->type]);
    if ((cpp = S_MRT_MSG_SUBTYPES[msg->type]) != NULL)
      printf ("/%s", cpp[msg->subtype]);
    printf("\n");
    Delete (stime);
  }

  while (len > 0) {
    UTIL_GET_NETLONG (prefix_long, cp);
    UTIL_GET_BYTE (mask, cp);
    UTIL_GET_BYTE (status, cp);
    UTIL_GET_NETLONG (originated, cp);
    UTIL_GET_SHORT (attrlen, cp);

    prefix = New_Prefix (AF_INET, (u_char *) &prefix_long, mask);
    
    peer = create_fake_peer ();
    bgp_munge_attributes (attrlen, cp, peer);

    t = msg->tstamp - originated;
    if (t / 3600 > 99)
      sprintf (date, "%02lddy%02ldhr", 
	       t / (3600 * 24), (t % (3600 * 24)) / 3600);
    else
      sprintf (date, "%02ld:%02ld:%02ld", 
	       t / 3600, (t / 60) % 60, t % 60);

    if (peer->attr) 
      printf ("B   20  %-15s %-15s   hme0  %s  %s\n",
	      date, prefix_toax (prefix), aspath_toa (peer->attr->aspath),
	      _sorigins[peer->attr->origin]);

    Delete (prefix);
    if (peer->attr) 
      bgp_deref_attr (peer->attr);
    Delete (peer);
    cp += attrlen;
    len -= attrlen;
    len -= 12;
  }
}


static bgp_peer_t *create_fake_peer () {
  bgp_peer_t *peer;

  peer = New (bgp_peer_t);
   peer->peer_id = 0L;
  peer->aliases = NULL;
  peer->sockfd = -1;
  peer->sockfd_in = -1;
  peer->sockfd_listen = -1;

  peer->dlist_in = peer->dlist_out = -1;
  peer->flist_in = peer->flist_out = -1;
  peer->route_map_in = peer->route_map_out = -1;

  peer->read_ptr = peer->buffer;
  peer->start_ptr = peer->buffer;
  peer->packet = NULL;
  peer->read_ptr_in = peer->buffer_in;

  peer->state = BGPSTATE_IDLE;

  peer->maximum_prefix = 0;

  peer->Start_Interval = -1;
  peer->ConnectRetry_Interval = -1;
  peer->KeepAlive_Interval = -1;
  peer->HoldTime_Interval = -1;

  return (peer);
}
